#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "font.h"
#include "glcd.h"
#include <avr/interrupt.h>

#define MAXREADS 3
#define MAXSAMPLES 128

#ifndef F_CPU
#define F_CPU 1600000UL
#endif

#include <stdbool.h>

int avgTemp[8] = {1, 1, 1, 1, 1, 1, 1, 1};
int minTemp[8] = {1, 1, 1, 1, 1, 1, 1, 1};
int maxTemp[8] = {1, 1, 1, 1, 1, 1, 1, 1};

int avgSound[MAXSAMPLES] = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};

int i,j,readcounter;


 
//**********************************************************
 
               
#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)
 
unsigned char LCD_buffer[16];
unsigned char LCD_bufferCopy[16];
char charToAdd ;
uint8_t ReceivedByte ;
bool startBuffering;
volatile bool bCharReceived;
int iCurrent = 0;
uint8_t elapsedTime=0;
bool showTemp = false;

 
void usart_init()
{
               
	UCSR0B = (1 << RXEN0) | (1 << TXEN0);   // Turn on the transmission and reception circuitry
	UCSR0C =  (1 << UCSZ00) | (1 << UCSZ01) | (1<<UPM01); // Use 8-bit character sizes and Even Parity Check

	UBRR0H = (BAUD_PRESCALE >> 8); // Load upper 8-bits of the baud rate value into the high byte of the UBRR register
	UBRR0L = BAUD_PRESCALE; // Load lower 8-bits of the baud rate value into the low byte of the UBRR register

}
 
uint8_t USART_vReceiveByte(void)
{
	// Wait until a byte has been received
	while((UCSR0A&(1<<RXC0)) == 0);
	// Return received data
	return UDR0;
}

void addTemp(int tempVal)
{
	int cnt;
	if(readcounter>=MAXREADS)
	{
	for (cnt=0;cnt<7;cnt++)
	{
	minTemp[cnt] = minTemp [cnt+1];
	avgTemp[cnt] = avgTemp[cnt+1];
	maxTemp[cnt] = maxTemp[cnt+1];
	}
	minTemp[cnt]=tempVal;
	avgTemp[cnt]=tempVal;
	maxTemp[cnt]=tempVal;
	readcounter = 0;
	}
	else if(readcounter==0)
	{
	minTemp[7]=tempVal;
	avgTemp[7]=tempVal;
	maxTemp[7]=tempVal;
	readcounter++;
	   }
	   else
	   {
	if(minTemp[7] > tempVal)
	minTemp[7] = tempVal;

	if(maxTemp[7] < tempVal)
	maxTemp[7] = tempVal;
  
	avgTemp[7]=(avgTemp[7]*readcounter+tempVal)/(readcounter+1);
	readcounter++;
	}   
}

void addSound(int soundVal)
	{
	int cnt;
	for (cnt=0;cnt<MAXSAMPLES-1;cnt++)
	{
	avgSound[cnt] = avgSound[cnt+1];
	}
	avgSound[cnt]=soundVal;  
}
 
 
ISR (USART0_RX_vect)
{

	int absTemp, absSound;
	char * tempStr;
	char * soundStr;
	int soundval;
	int temperature;

	UDR0=0;//clear UART buffer
	ReceivedByte = USART_vReceiveByte();//receive data

	if(((ReceivedByte >31) && (ReceivedByte < 91)) || (ReceivedByte == 124))
	{
		if (ReceivedByte=='#') 
		startBuffering=true; 


		if (ReceivedByte=='*')
		{
			showTemp = !showTemp;


			LCD_buffer[iCurrent]='\0';   // To append a null character to the string received.

			tempStr = strtok(LCD_buffer,"|");
			soundStr = strtok(NULL,"|");

			temperature = atoi(tempStr);   // atoi: To convert String to int
			soundval = atoi(soundStr);

			absTemp = (temperature+20)/4;  // To display the graphical values of the temperature properly
			if (absTemp > 32)              // To ensure that the graphical lines doesn't overlap with the text above
			absTemp = 32;

			absSound= (soundval-6)/2;  // To display the graphical values of the temperature properly
			if (absSound > 32)              // To ensure that the graphical lines doesn't overlap with the text above
			absSound = 32;

			addTemp(absTemp);
			addSound(absSound);

			glcd_clear();
			goto_xy(0,0);

			if (showTemp)
			{
				glcd_write_string("Temp : ");
				glcd_write_string(tempStr);
				glcd_write_string("F");

				goto_xy(0,2);
				if (temperature >= 80)
				glcd_write_string("HIGH");
				else if (temperature <= 60)
				glcd_write_string("LOW");
				else
				glcd_write_string("NORMAL");


				//  h_line(0,59,128,1);

				for (i=0;i<8;i++)
				{
					v_line(i*16+6,64-minTemp[i],minTemp[i],1);
					v_line(i*16+7,64-avgTemp[i],avgTemp[i],1);       
					v_line(i*16+8,64-avgTemp[i],avgTemp[i],1);
					v_line(i*16+9,64-maxTemp[i],maxTemp[i],1);
					if(i>0)
					{
						line(((i-1)*16)+7,64-avgTemp[i-1],(i*16)+7,64-avgTemp[i],1);
					}
				}

			}
			else
			{
				glcd_write_string("Sound : ");       
				glcd_write_string(soundStr);       
				glcd_write_string("db");

				goto_xy(0,2);
				if (soundval >= 40)
				glcd_write_string("HIGH");
				else if (soundval <= 20)
				glcd_write_string("LOW");
				else
				glcd_write_string("NORMAL");


				// h_line(0,59,128,1);

				for (i=0;i<MAXSAMPLES;i++)
				{         	
					v_line(i,64-avgSound[i],avgSound[i],1);       
				}
			}  

			LCD_buffer[0]= '\0';
			startBuffering = false;
			iCurrent=0;

		}

		else if (ReceivedByte!='#')
		{
			LCD_buffer[iCurrent++] = ReceivedByte;
		}

	}
}

void init_devices (void)
{
 cli();                    //Clears the global interrupts
 port_init();     			//initialize ports
 sei();                    //Enables the global interrupts
}




int main ( void )
{
	init_devices();  
    glcd_on();    //initialize glcd
    usart_init();


    TCCR1B |= (1 << CS10) | (1 << CS12); // Set up timer at Fcpu/1024 = 15625
    
	startBuffering = false;
   
    UCSR0B |= (1 << RXCIE0 ); //Enable the USART Recieve Complete interrupt ( USART_RXC )

    while(1) // Loop forever
    {
	 	
	}
}
