#define E_DELAY    10
#define DATAPORT   PORTC
#define CONTROLPORT  PORTA
#define CONTROLADDR  DDRA
#define DATADDR   DDRC
#define DATAPIN   PINC
#define chip_selec_1 (1<<PA0) //chip select for controller 1
#define chip_selec_2 (1<<PA1) //chip select for controller 2
#define RS     (1<<PA2) //register select
#define RW     (1<<PA3) //read/write
#define EN     (1<<PA4) //enable
#define RST     (1<<PA5) //reset

unsigned char cy=0,cx=0;

//------------------------------------------------------------------------------------------------------------
void port_init(void)
{   
 DATAPORT  = 0xFF; 
 DATADDR   = 0xFF; //initialize as output
 CONTROLPORT = 0x00;
 CONTROLADDR = 0xFF; //initialize as output
}

//-----------------------------------------------------------------------------------------------------------
void trigger_pulse()
{
 CONTROLPORT |= EN; //EN high
   _delay_us(E_DELAY);
   CONTROLPORT &= ~EN; //EN low
   _delay_us(E_DELAY);
}
//------------------------------------------------------------------------------------------------------------
void glcd_on()
{
 CONTROLPORT &= ~RS;           //RS low --> command
   CONTROLPORT &= ~RW;           //RW low --> write
   
 DATAPORT = 0x3F;           //ON command


 CONTROLPORT  |=  chip_selec_1;      //select chip 1
 CONTROLPORT  &= ~chip_selec_2;
 
 CONTROLPORT &= ~RST;    //logic low on reset
 CONTROLPORT |= RST;     //logic high on reset

 trigger_pulse();

 CONTROLPORT  &= ~chip_selec_1;      //select chip 2
 CONTROLPORT  |=  chip_selec_2;
 
 CONTROLPORT &= ~RST;    //logic low on reset
 CONTROLPORT |= RST;     //logic high on reset
 trigger_pulse();
   
}           
//------------------------------------------------------------------------------------------------------------
void glcd_off()
{
 CONTROLPORT &= ~RS;       //RS low --> command
 CONTROLPORT &= ~RW;       //RW low --> write
 DATAPORT = 0x3E;        //OFF command

 CONTROLPORT  |=  chip_selec_1;        //select chip 1
 CONTROLPORT  &= ~chip_selec_2;
 trigger_pulse();

 CONTROLPORT  &= ~chip_selec_1;        //selcet chip 2
 CONTROLPORT  |=  chip_selec_2;
 
 trigger_pulse(); 
  
}     
//-------------------------------------------------------------------------------------------------------------
void set_start_line(unsigned char line)
{
  CONTROLPORT &= ~RS;     //RS low --> command
  CONTROLPORT &= ~RW;     //RW low --> write
   
  DATAPORT = 0xC0 | line;        //Set Start Line command

  CONTROLPORT  |=  chip_selec_1;        //select chip 1
  CONTROLPORT  &= ~chip_selec_2;
  trigger_pulse();

  CONTROLPORT  &= ~chip_selec_1;        //select chip 2
  CONTROLPORT  |=  chip_selec_2;
  
  trigger_pulse();
}

//--------------------------------------------------------------------------------------------------------------
void goto_col(unsigned int x)
{
   unsigned char pattern;
   CONTROLPORT &= ~RS;     //RS low --> command
   CONTROLPORT &= ~RW;     //RW low --> write
   cy = x;
   if(x<64)                 //left section
   {
      CONTROLPORT |= chip_selec_1;  //deselect chip 1
      CONTROLPORT &= ~chip_selec_2;  //select chip 2

      pattern = x;          //put column address on data port
   }
   else                     //right section
   {
    CONTROLPORT &= ~chip_selec_1; //deselct chip 1
    CONTROLPORT |= chip_selec_2;  //select chip 2

    pattern = x-64;       //put column address on data port
   }    
   pattern = (pattern | 0x40);    //Command format
   DATAPORT = pattern; 
   trigger_pulse();     
}   
//-------------------------------------------------------------------------------------------------------------
void goto_row(unsigned int y)
{    
   unsigned char pattern;
   CONTROLPORT &= ~RS;  //RS low --> command
   CONTROLPORT &= ~RW;  //RW low --> write
   pattern = (y | 0xB8); //put row address on data port set command
   cx = y;
   DATAPORT = pattern;   
   trigger_pulse();             
}
//--------------------------------------------------------------------------------------------------------------
void goto_xy(unsigned int x,unsigned int y)
{    
    goto_col(x); //column
    goto_row(y); //row
}  
//--------------------------------------------------------------------------------------------------------------
void glcd_write(unsigned char b)
{
 CONTROLPORT |= RS; //RS high --> data
 CONTROLPORT &= ~RW; //RW low --> write
 DATAPORT = b;     //put data on data port
   _delay_us(1);
   trigger_pulse();
}
//--------------------------------------------------------------------------------------------------------------
void glcd_clrln(unsigned char line)
{
    int i;
    goto_xy(0,line);      //At start of line of left side
    goto_xy(64,line);     //At start of line of right side

    CONTROLPORT |= chip_selec_1;

    for(i=0;i<64;i++)
        glcd_write(0);
}
//---------------------------------------------------------------------------------------------------------------
void glcd_clear()
{
   int i;
   for(i=0;i<8;i++)
    glcd_clrln(i);
}
//---------------------------------------------------------------------------------------------------------------   
unsigned char glcd_read(unsigned char column)
{
    unsigned char read_data = 0; //Read data here
    DATADDR = 0x00;     //Input 
    
    CONTROLPORT |= RW; //Read
    CONTROLPORT |= RS; //Data

    if(column<64)
    {
     CONTROLPORT |= chip_selec_1;  //Enable chip_selec_1
     CONTROLPORT &= ~chip_selec_2;  //Disable chip_selec_2
    }
    else
    {
     CONTROLPORT |= chip_selec_2;  //Enable chip_selec_2
     CONTROLPORT &= ~chip_selec_1; //Disable chip_selec_1
    }
    _delay_us(1);          //tasu
    CONTROLPORT |= EN;   //Latch RAM data into ouput register
    _delay_us(1);          //twl + tf
    
    //Dummy read
    CONTROLPORT &= ~EN;   //Low Enable
    _delay_us(20);         //tf + twl + chineese error
    
    CONTROLPORT |= EN;   //latch data from output register to data bus
    _delay_us(1);          //tr + td(twh)
                                  
    read_data = DATAPIN;     //Input data
    CONTROLPORT &= ~EN;   //Low Enable to remove data from the bus
    _delay_us(1);          //tdhr
    DATADDR = 0xFF;       //Output again
    return read_data;      
}
//-----------------------------------------------------------------------------------------------------------------
void draw_point(unsigned int x,unsigned int y,unsigned char color) //x = column,y = row
{
    unsigned char pattern = 0;
    goto_xy(x,(int)(y/8));   //x is column y is row y/8 gives constant row value
    switch (color)
    {               
        case 0:         //Blank spot                        
            pattern = ~(1<<(y%8)) & glcd_read(x);                  
        break;    
        case 1:         //Dark spot            
            pattern = (1<<(y%8)) | glcd_read(x);
        break;
    }           
    goto_xy(x,(int)(y/8));                                
    glcd_write(pattern);
}
//------------------------------------------------------------------------------------------------------------------
void h_line(unsigned int x,unsigned int y,unsigned char l,unsigned char c)
{
    int i;
    for(i=x; i<(l+x); i += 1)
        draw_point(i,y,c);
}
//------------------------------------------------------------------------------------------------------------------
void v_line(unsigned int x,unsigned int y,signed int l,unsigned char c)
{
    unsigned int i;
    for(i=y; i<(y+l); i += 1)
        draw_point(x,i,c);
}

//------------------------------------------------------------------------------------------------------------------

void line(unsigned int x1,unsigned int y1,unsigned int x2, unsigned int y2,unsigned char c)
{
int dx, dy, p, end, ydir;
float x, y;
   ydir = 1; 
   if (y1>y2)
      ydir = -1;
   dx = abs(x1 - x2);
   dy = abs(y1 - y2);
   p = 2 * dy - dx;
   if(x1 > x2)
   {
  x = x2;
  y = y2;
  end = x1;
   }
   else
   {
  x = x1;
  y = y1;
  end = x2;
   }
   draw_point(x,y,c);
   while(x < end)
   {
  x = x + 1;
  if(p < 0)
  {
        p = p + 2 * dy;
  }
  else
  {
        y = y + ydir;
        p = p + 2 * (dy - dx);
  }
  draw_point(x,y,c);
   }
}

//------------------------------------------------------------------------------------------------------------------
void glcd_write_char(char charToWrite) //character to display
{
 int i;
 if(cy<64)
 { 
     CONTROLPORT |= chip_selec_1; //select chip 1
     CONTROLPORT &= ~chip_selec_2; //deselect chip 2
 }
 else
 {
  CONTROLPORT &= ~chip_selec_1; //deselct chip 1
     CONTROLPORT |= chip_selec_2; //select chip 2
 }
  
 for(i=0;i<8;i++)
 {
       goto_xy(cy,cx); //cy=column,cx=row
    glcd_write(pgm_read_byte(&(font[(8*charToWrite)+i]))); //address of character is provided in program read byte to read the bytes for character
    cy++;
  }
}
//-------------------------------------------------------------------------------------------------------------------
void glcd_write_string(char *stringToWrite)
{
 while(*stringToWrite)
   glcd_write_char(*stringToWrite++);
}
//-------------------------------------------------------------------------------------------------------------------
void lcd_image(const unsigned char *string)
{ 
 unsigned char j=0,i=0;
 goto_xy(0,0);
 while(i<8)   //row
 {
  while(j<128) //column
  {
   glcd_write((*string)); //write byte from array of image
   string++;
   if(j==63)
   {
    goto_xy(64,i);  //select second controller
   }
   j++; 
  }
  j=0;
  i++;
  goto_xy(0,i);
 }
}
